import re
import sys

import requests
import threading
from colorama import Fore, init

init()


# Class to manage and store url information
class Link:
    def __init__(self, url):
        self.linkUrl = url
        self.linkStatus = None
        self.linkValid = None
        self.linkInfo = ""

    def checkStatus(self, option):

        try:
            self.linkStatus = requests.head(self.linkUrl).status_code
            if self.linkStatus == 200:
                self.linkValid = "good"
            elif self.linkStatus == 400 or self.linkStatus == 404:
                self.linkValid = "bad"
            else:
                self.linkValid = "unknown"
        except requests.exceptions.RequestException:
            self.linkValid = "unknown"
            self.linkStatus = "failed to establish a connection"
        if option == "json":
            self.linkInfo = (
                '{ "url": \''
                + self.linkUrl
                + '\', "status":'
                + str(self.linkStatus)
                + " }"
            )
        else:
            self.linkInfo = (
                self.linkUrl
                + " is a "
                + self.linkValid
                + " link with a HTTP status of "
                + str(self.linkStatus)
            )

        if self.linkStatus == 200:
            if option != "bad":
                print(Fore.GREEN + self.linkInfo)
        elif self.linkStatus == 400 or self.linkStatus == 404:
            if option != "good":
                print(Fore.RED + self.linkInfo)
        else:
            if option != "good":
                print(Fore.YELLOW + self.linkInfo)


# Class to manage and store file information
class TextFile:
    def __init__(self, filePath):
        # try to read and store file information, catch fileNotFound and IO errors
        try:
            self.fileText = open(filePath, "r").read()
            self.fileUrls = set(
                re.findall(
                    "(http|ftp|https)://([\w_-]+(?:(?:\.[\w_-]+)+))([\w.,@?^=%&:/~+#-]*[\w@?^=%&/~+#-])?",
                    self.fileText,
                )
            )
            self.fileLinks = [
                Link((url[0] + "://" + url[1] + url[2])) for url in self.fileUrls
            ]
        except FileNotFoundError:
            self.fileText = None
            print("File was not found")
        except IOError:
            print("File was unable to be read")
            self.fileText = None

    def checkLinkStatuses(self, option):
        # use multithreading for http requests
        self.fileThreads = [
            threading.Thread(target=url.checkStatus(option)) for url in self.fileLinks
        ]
        for thread in self.fileThreads:
            thread.start()
        for thread in self.fileThreads:
            thread.join()

    def compareLinks(self, link, pattern):
        # try to read and store file information, catch fileNotFound and IO errors
        self.match = False
        for url in pattern:
            if re.search(url.linkUrl, link):
                self.match = re.search(url.linkUrl, link)
                break


# Class to manage and store ignore file information
class IgnoreFile:
    def __init__(self, ignoreFilePath):
        # try to read and store file information, catch fileNotFound and IO errors
        try:
            path = ignoreFilePath[0]
            self.fileText = open(path, "r").read()
            self.fileComments = set(re.findall("#.*", self.fileText))
            self.fileUrl = set(
                re.findall(
                    "(?!# )(http|https)://([\w_-]+(?:(?:\.[\w_-]+)+))([\w.,@?^=%&:/~+#-]*[\w@?^=%&/~+#-])?",
                    self.fileText,
                )
            )
            self.fileInvalidUrl = set(
                re.findall(
                    "(?!# )(?!http|https)(?!://)([\w_-]+(?:(?:\.[\w_-]+)+))([\w.,@?^=%&:/~+#-]*[\w@?^=%&/~+#-])?",
                    self.fileText,
                )
            )
            self.fileLink = []
            if self.fileUrl:
                self.fileLink = [
                    Link((url[0] + "://" + url[1] + url[2])) for url in self.fileUrl
                ]
            elif self.fileInvalidUrl:
                print(
                    "The URL provided is invalid. Must begin with https:// or http://"
                )
                sys.exit(-1)
        except FileNotFoundError:
            self.fileText = None
            print("File was not found")
        except IOError:
            print("File was unable to be read")
            self.fileText = None



from .. import fileReader


# Test if fileNotFound error is raised and chosen output is printed
def test_fileNotFound(capsys):
    fileReader.TextFile("doesntExist.txt")
    captured = capsys.readouterr()
    assert captured.out == "File was not found\n"


def test_urlParse():
    filePath = "tests/testFile.txt"
    testUrl = "https://gist.github.com/chrispinkney/069b09d2da5b9f7b73347d13ba3c32e7.js"
    fileUrl = []
    testFile = fileReader.TextFile(filePath)
    for url in testFile.fileUrls:
        fileUrl.append(url[0] + "://" + url[1] + url[2])
    assert fileUrl[0] == testUrl


def test_linkObject_created():
    filePath = "tests/testFile.txt"
    testFile = fileReader.TextFile(filePath)
    for link in testFile.fileLinks:
        assert isinstance(link, fileReader.Link)


def test_linkObject_URL():
    filePath = "tests/testFile.txt"
    testFile = fileReader.TextFile(filePath)
    testUrl = "https://gist.github.com/chrispinkney/069b09d2da5b9f7b73347d13ba3c32e7.js"
    assert testFile.fileLinks[0].linkUrl == testUrl



import pytest
import requests
from .. import fileReader


def test_link_status404(requests_mock):
    testUrl = "https://www.google.ca/"
    requests_mock.head(testUrl, status_code=404)
    testLink = fileReader.Link(testUrl)
    testLink.checkStatus("All")
    assert testLink.linkStatus == 404


def test_link_status200(requests_mock):
    testUrl = "https://www.google.ca/"
    requests_mock.head(testUrl, status_code=200)
    testLink = fileReader.Link(testUrl)
    testLink.checkStatus("All")
    assert testLink.linkStatus == 200


def test_link_exception(requests_mock):
    testUrl = "https://www.google.ca/"
    requests_mock.head(testUrl, exc=requests.exceptions.RequestException)
    testLink = fileReader.Link(testUrl)
    testLink.checkStatus("All")
    assert testLink.linkStatus == "failed to establish a connection"


def test_link_statusUnknown(requests_mock):
    testUrl = "https://www.google.ca/"
    requests_mock.head(testUrl, status_code=500)
    testLink = fileReader.Link(testUrl)
    testLink.checkStatus("All")
    assert testLink.linkValid == "unknown"


def test_message_good(requests_mock):
    testUrl = "https://www.google.ca/"
    requests_mock.head(testUrl, status_code=200)
    testLink = fileReader.Link(testUrl)
    testLink.checkStatus("All")
    assert testLink.linkInfo == (testUrl + " is a good link with a HTTP status of 200")


def test_message_bad(requests_mock):
    testUrl = "https://www.google.ca/"
    requests_mock.head(testUrl, status_code=404)
    testLink = fileReader.Link(testUrl)
    testLink.checkStatus("All")
    assert testLink.linkInfo == (testUrl + " is a bad link with a HTTP status of 404")


def test_message_json(requests_mock):
    testUrl = "https://www.google.ca/"
    requests_mock.head(testUrl, status_code=404)
    testLink = fileReader.Link(testUrl)
    testLink.checkStatus("json")
    assert testLink.linkInfo == (
        '{ "url": \'' + testUrl + '\', "status":' + str(404) + " }"
    )


def test_option_good(requests_mock, capsys):
    testUrl = "https://www.google.ca/"
    requests_mock.head(testUrl, status_code=200)
    testLink = fileReader.Link(testUrl)
    testLink.checkStatus("good")
    captured = capsys.readouterr()
    assert captured.out == (
        "\x1b[32m" + testUrl + " is a good link with a HTTP status of 200\n"
    )
    requests_mock.head(testUrl, status_code=404)
    testLink = fileReader.Link(testUrl)
    testLink.checkStatus("good")
    captured = capsys.readouterr()
    assert captured.out == ""


def test_option_bad(requests_mock, capsys):
    testUrl = "https://www.google.ca/"
    requests_mock.head(testUrl, status_code=404)
    testLink = fileReader.Link(testUrl)
    testLink.checkStatus("bad")
    captured = capsys.readouterr()
    assert captured.out == (
        "\x1b[31m" + testUrl + " is a bad link with a HTTP status of 404\n"
    )
    requests_mock.head(testUrl, status_code=200)
    testLink = fileReader.Link(testUrl)
    testLink.checkStatus("bad")
    captured = capsys.readouterr()
    assert captured.out == ""
    requests_mock.head(testUrl, status_code=500)
    testLink = fileReader.Link(testUrl)
    testLink.checkStatus("bad")
    captured = capsys.readouterr()
    assert captured.out == (
        "\x1b[33m" + testUrl + " is a unknown link with a HTTP status of 500\n"
    )



import argparse
import fileReader
import threading
import sys
from termcolor import colored
import colorama


parser = argparse.ArgumentParser(
    description="Determine status of link in file",
    epilog="""The CLI takes a file path (relative or absolute)
                                        and reads and parses it for urls to check their http status""",
)

# take file names as an argument on command line
parser.add_argument("files", nargs="+")
# create command line option to output live links only
parser.add_argument("-g", "--good", action="store_true", help="Outputs only live links")
# create command line option to output dead and unknown status links only
parser.add_argument("-b", "--bad", action="store_true", help="Outputs only dead links")
# create command line option to output all links
parser.add_argument("-a", "--all", action="store_true", help="Outputs all links")
# create command line option for general information
parser.add_argument(
    "-in",
    "--info",
    action="store_true",
    help="Outputs overall information about links in file, i.e how many live or dead links there are",
)
# create command line option to output in JSON format
parser.add_argument(
    "-j", "--json", action="store_true", help="Outputs links in json format"
)
# create command line option to ignore specified link patterns
parser.add_argument(
    "-i",
    "--ignore",
    nargs=1,
    help="Outputs the links that do not match the provided pattern",
)


def processFile(file, option):
    colorama.init()
    urlFile = fileReader.TextFile(file)
    processLinks(file, urlFile, option)


def processFileWithIgnore(file, pattern, option):
    colorama.init()
    urlFile = fileReader.TextFile(file)
    for link in urlFile.fileLinks:
        urlFile.compareLinks(link.linkUrl, pattern)
        if urlFile.match:
            urlFile.fileLinks.remove(link)
    processLinks(file, urlFile, option)


def processLinks(file, urlFile, option):
    if urlFile.fileText is not None:
        urlFile.checkLinkStatuses(option)
    # info option, get counts for link status types
    if option == "info":
        deadLinks = 0
        liveLinks = 0
        unknownLinks = 0
        for url in urlFile.fileLinks:
            if url.linkValid == "good":
                liveLinks += 1
            if url.linkValid == "unknown":
                unknownLinks += 1
            if url.linkValid == "bad":
                deadLinks += 1
        print(
            "The file "
            + file
            + " has "
            + colored(str(deadLinks) + " dead links ", "red")
            + " and "
            + colored(str(liveLinks) + " live links", "green")
        )
        print(
            "There are also " + colored(str(unknownLinks) + " unknown links", "yellow")
        )


# take command line files and create thread for each file
def processArguments(argsNormal, option):
    for arg in argsNormal:
        threading.Thread(target=processFile(arg, option)).start()


# take command line files and create thread for each file, while ignoring specified links
def processArgumentsWithIgnore(argsWithIgnore, pattern, option):
    for arg in argsWithIgnore:
        threading.Thread(target=processFileWithIgnore(arg, pattern, option)).start()


if __name__ == "__main__":
    # run argparse -help option if no arguments entered on command line
    if len(sys.argv) == 1:
        parser.print_help()
        sys.exit(1)


def filterArguments(args):
    if args.ignore:
        ignoreFile = fileReader.IgnoreFile(args.ignore)
        if args.good:
            processArgumentsWithIgnore(args.files, ignoreFile.fileLink, "good")
        elif args.bad:
            processArgumentsWithIgnore(args.files, ignoreFile.fileLink, "bad")
        elif args.info:
            processArgumentsWithIgnore(args.files, ignoreFile.fileLink, "info")
        elif args.json:
            processArgumentsWithIgnore(args.files, ignoreFile.fileLink, "json")
        else:
            processArgumentsWithIgnore(args.files, ignoreFile.fileLink, "all")
    else:
        if args.good:
            processArguments(args.files, "good")
        elif args.bad:
            processArguments(args.files, "bad")
        elif args.info:
            processArguments(args.files, "info")
        elif args.json:
            processArguments(args.files, "json")
        else:
            processArguments(args.files, "all")


# parse arguments
args = parser.parse_args()
filterArguments(args)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="DeadOrNot",
    version="0.9",
    author="Abdul Abdi",
    author_email="aabdi39@myseneca.ca",
    description="A command-line tool for reporting on the status of links in a file",
    url="https://github.com/AbdulMAbdi/deadOrNot",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "colorama",
        "requests",
        "termcolor",
        "pytest",
        "pre-commit",
        "black",
        "flake8",
        "requests-mock",
        "pytest-cov",
    ],
    packages=setuptools.find_packages(),
    entry_points={
        "console_scripts": [
            "deadOrNot = src.deadOrNot:main",
        ]
    },
    python_requires=">=3.6",
)



import pytest
from .. import fileReader


def test_ignoreFileInvalid(capsys):
    filePath = ["tests/testInvalidIgnore.txt"]
    with pytest.raises(SystemExit):
        fileReader.IgnoreFile(filePath)
    captured = capsys.readouterr()
    assert (
        captured.out
        == "The URL provided is invalid. Must begin with https:// or http://\n"
    )


def test_ignoreFileValid():
    filePath = ["tests/testInvalidCommentUrl.txt"]
    testIgnore = fileReader.IgnoreFile(filePath)
    assert (
        testIgnore.fileLink[0].linkUrl == "https://www.google.com"
        or "https://www.google.com/"
    )


def test_ignoreFileComment():
    filePath = ["tests/testComment.txt"]
    testIgnore = fileReader.IgnoreFile(filePath)
    assert testIgnore.fileComments == {"# 1. Empty file, nothing will be ignored"}


# Test if fileNotFound error is raised and chosen output is printed
def test_ignoreFileNotFound(capsys):
    fileReader.IgnoreFile("doesntExist.txt")
    captured = capsys.readouterr()
    assert captured.out == "File was not found\n"
